/**
 * 
 */
package drawing;

import java.awt.Point;
import java.io.Serializable;
import java.util.ArrayList;


/**
 * Klasa reprezentujca krzyw. Implementuje interfejs Serializable. Jest przesyana przez sie.
 * Zawiera list punktw, ktre si na ni skadaj, kolor i grubo (klasa Pen).
 */
public class Curve implements Serializable{
	/**
	 * Lista punktw skadajcych si na t krzyw.
	 */
	private ArrayList points = new ArrayList();	
	/**
	 * Pozycja na licie punktw. Do iteracji.
	 */
	private int actualPos=0;
	/**
	 * Autor krzywej.
	 */
	private String author = null;
	/**
	 * Instrukcja "podczepiana" do krzywej w celu rozesania do wyszystkich klientw.
	 * Moe by tylko jedna: clear, czyli czyszczenie kartki. 
	 */
	private String instruction = null;
	/**
	 * Wygld krzywej.
	 */
	private Pen pen;
	
	/**
	 * Konstruktor.
	 * @param author Autor krzywej.
	 */
	public Curve(String author) {
		super();
		// TODO Auto-generated constructor stub
		this.author = author;
	}
	
	/**
	 * Konstruktor bezargumentowy. Autor pozostaje nieustalony.
	 *
	 */
	public Curve(){}

	/**
	 * Dodaje punkt do krzywej.
	 * @param p
	 */
	public void addPoint(Point p){
		points.add(p);
	}
	
	/**
	 * Metoda do iterowania po punktach z ktrych skada si krzywa. Jeli nie ma wicej punktw 
	 * podnosi wyjtek (i resetuje si, tzn. mona iterowa po wszystkich punktach od pocztku). 
	 * @return Zwraca kolejny punkt krzywej.
	 * @throws NoMoreLeftException
	 */
	public Point getNextPoint() throws NoMoreLeftException{
		try {
			return (Point)points.get(actualPos++);
		} catch (IndexOutOfBoundsException e) {			 
			actualPos=0;
			throw new NoMoreLeftException();
		}
	}
	
	/**
	 * Zwraca ilo punktw w krzywej.
	 * @return Il punktw z ktrych jest zbudowana krzywa.
	 */
	public int getPointCount(){
		return points.size();
	}

	/**
	 * @return Autor krzywej.
	 */
	public String getAuthor() {
		return author;
	}

	/**
	 * Ustawianie autora krzywej (serwer mu jej nie odele).
	 * @param author Autor krzywej
	 */
	public void setAuthor(String author) {
		this.author = author;
	}

	/**
	 * Daj pisak.
	 * @return Pisak uzyty przy rysowaniu tej krzywej.
	 */
	public Pen getPen() {
		return pen;
	}

	/**
	 * Ustaw pisak.
	 * @param pen Pisak.
	 */
	public void setPen(Pen pen) {
		this.pen = pen;
	}
	
	
	/**
	 * Daj instrukcj. W raz z krzywymi mozna dodatkowo przesya instrukcje dla innych klientw bd serwera.
	 * Obecnie wycznie instrukcja clear (nowa kartka).
	 * @return Instrukcja.
	 */
	public String getInstruction() {
		return instruction;
	}

	/**
	 * Ustaw instrucj.
	 * @param instruction Instrukcja
	 */
	public void setInstruction(String instruction) {
		this.instruction = instruction;
	}
		
}
