package drawingclient;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.RemoteException;

import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.border.BevelBorder;

import drawing.Pen;
import drawingserver.DrawingServer;

/**
 * 
 * Gwny Panel aplikacji/apletu, z ptnem do rysowania i interfejsem graficznym.
 * Obsuguje komunikacj midzy interfejsem a panelem rysujcym. 
 * Inicjuje panel odpwiedzialny za rysowanie, ktry nawizuje poczenie z serwerem.
 */

public class JPanel_DrawingDrawing extends javax.swing.JPanel implements ErrorShowing{
	
	/**
	 * Panel na ktrym odbywa si rysowanie.
	 */
	private JPanel_Drawer jPanel_Drawer;

	
	/**
	 * Bank krzywych.
	 */
	private CurveBank curveBank = new CurveBank();
	
	/**
	 * Przybornik z narzdziami.
	 */
	private JPanel_ToolBox jPanel_ToolBox;

	/**
	 * Inicjalizacja interfejsu uytkownika.
	 *
	 */
	public JPanel_DrawingDrawing() {
		super();
		curveBank.addErrorShower(this);
		initGUI();
		
		//jPanel_Drawer.setUiData(jPanel_ToolBox);
	}

	/**
	 * Ustawianie wskanika na interfejs DrawingServer. Inicjalizacja banku krzywych (CurveBank).
	 * @param drawingServer Serwer 
	 * @throws RemoteException
	 */
	public void setDrawingServer(DrawingServer drawingServer)
			throws RemoteException {
		curveBank.init(drawingServer);		
		jPanel_Drawer.setCurveBank(curveBank);
	}
	
	/**
	 * Pokaz bad. Bd zostaje pokazany w modalnym okienku a aplikacja zamknita.
	 * @param e Wyjtek.
	 */
	public void showError(Exception e){
		//System.out.println(e)
		e.printStackTrace();
		JOptionPane.showMessageDialog(this, e.getMessage()+"\nThis is fatal. The application will close.", "Error", JOptionPane.ERROR_MESSAGE);
		System.exit(-1);
	}

	/**
	 * Metoda inicjalizujca interfejs uytkownika.
	 *
	 */
	private void initGUI() {
		try {
			BorderLayout thisLayout = new BorderLayout();
			this.setLayout(thisLayout);

			jPanel_ToolBox = new JPanel_ToolBox();
			jPanel_ToolBox.setBorder(new BevelBorder(BevelBorder.RAISED));
			this.add(jPanel_ToolBox, BorderLayout.EAST);
			jPanel_ToolBox.setPreferredSize(new Dimension(150,400));

			jPanel_Drawer = new JPanel_Drawer();
			jPanel_Drawer.setBorder(new BevelBorder(BevelBorder.RAISED));
			this.add(jPanel_Drawer, BorderLayout.CENTER);
			
			jPanel_ToolBox.addPropertyChangedListener(new PropertyChangeListener(){
				public void propertyChange(PropertyChangeEvent evt) {
					if(evt.getPropertyName().equals("pen")){
						Pen p = (Pen)evt.getNewValue();
						jPanel_Drawer.setPen(p.deepCopy());
					}
					if(evt.getPropertyName().equals("aa")){
						Boolean b = (Boolean)evt.getNewValue();
						jPanel_Drawer.setAntialiased((boolean)b.booleanValue());
					}
					if(evt.getPropertyName().equals("clear")){
						jPanel_Drawer.clearSheetOfPaper();
					}
				}				
			});
			
			jPanel_Drawer.setPen(jPanel_ToolBox.getPen());
			jPanel_Drawer.setAntialiased(jPanel_ToolBox.isAntialiased());
					

		} catch (Exception e) {
			showError(e);
		}
	}	
	
	/**
	 * @return JMenuBar do wykorzystania przez JApplet lub JFrame. Pozwala na zpisywanie i wczytywanie
	 * obrazkw z dysku.
	 */
	public JMenuBar createJMenuBar(){
		JMenuBar menuBar = new JMenuBar();
		JMenu mFile = new JMenu("File");
		
		JMenuItem iSave = new JMenuItem("Save picture");
		
		iSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();		
				fc.setFileFilter(new PicFileFilter());
				if(fc.showSaveDialog((Component)e.getSource())==fc.APPROVE_OPTION){
					try {
						FileOutputStream out = new FileOutputStream(fc.getSelectedFile());
						curveBank.saveToStream(out);
						out.close();						
					} catch (IOException e1) {						 
						e1.printStackTrace();
					}
				}
			}
		});
		
		JMenuItem iLoad = new JMenuItem("Load picture");
		
		iLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();	
				fc.setFileFilter(new PicFileFilter());
				if(fc.showOpenDialog((Component)e.getSource())==fc.APPROVE_OPTION){
					try {
						FileInputStream in = new FileInputStream(fc.getSelectedFile());
						curveBank.loadFromStream(in);
						jPanel_Drawer.repaint();
						in.close();
					} catch (IOException e1) {						 
						e1.printStackTrace();
					}
				}
			}
		});
		
		mFile.add(iSave);
		mFile.add(iLoad);
		menuBar.add(mFile);
		return menuBar;
	}	

}
