package drawingclient;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import drawing.Pen;

/**
 * Bean do okrelania koloru i gruboci kreski. Generuje zdarzenie PropertyChanged
 * jeli uytkownik zmieni parametry pisaka. 
 */
public class JPanel_PenProporties extends javax.swing.JPanel {	

	/**
	 * Aktualnie wskazany przez uytkownika pisak.
	 */
	private Pen pen = new Pen();	

	/**
	 * Konstruktor. Inicjalizacja komponentu.
	 *
	 */
	public JPanel_PenProporties() {
		try {
			initGUI();
			setLineWidth(2);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Metoda inicjalizujca komponenty.
	 * @throws Exception
	 */
	private void initGUI() throws Exception {
		// this.setBorder(BorderFactory.createLineBorder(Color.black));
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		jButton_ColorPicker.setText("Color");
		jLabel_LineWidth.setText("LineWidth:");
		jSlider_LineWidthSetter.setMaximum(25);
		jSlider_LineWidthSetter.setMinimum(1);
		jSlider_LineWidthSetter.setValue(2);

		//add(Box.createVerticalStrut(20));
		add(jButton_ColorPicker);
		jButton_ColorPicker.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color c = JColorChooser.showDialog((Component) e.getSource(),
						"Choose color", pen.getColor());
				setColor(c);
			}
		});
		add(Box.createVerticalStrut(10));
		add(jLabel_LineWidth);
		add(Box.createVerticalStrut(10));
		add(jSlider_LineWidthSetter);
		jSlider_LineWidthSetter.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				setLineWidth(jSlider_LineWidthSetter.getValue());
			}
		});
		add(Box.createVerticalStrut(20));
	}

	/**
	 * Okrel grubo linii.
	 * @param w Grubo linii.
	 */
	public void setLineWidth(int w) {
		jLabel_LineWidth.setText("Line width: " + w);
		jSlider_LineWidthSetter.setValue(w);
		pen.setLineWidth(w);
		
		propertyChangeSupport.firePropertyChange("pen", null, pen);
	}
	
	/**
	 * Okrel kolor.
	 * @param c Kolor.
	 */
	public void setColor(Color c){
		jButton_ColorPicker.setForeground(c);
		pen.setColor(c);
		
		propertyChangeSupport.firePropertyChange("pen", null, pen);
		//somethingChanged.actionPerformed(new ActionEvent(this, "", "");
	}

	/**
	 * Guzik pokazujcy ColorPicker
	 */
	private JButton jButton_ColorPicker = new JButton();

	/**
	 * Napis informujcy o ustawionej gruboci linii.
	 */
	private JLabel jLabel_LineWidth = new JLabel();

	/**
	 * Suwak do ustawiania gruboci linii.
	 */
	private JSlider jSlider_LineWidthSetter = new JSlider();

	/**
	 * Daj pisak.
	 * @return Pisak 
	 */
	public Pen getPen() {
		return pen;
	}
	
	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
		
	/**
	 * Dodaj listener nasuchujcy zmian paramterw pisaka (koloru bd gruboci).
	 * @param pcl PropertyChangeListener
	 */
	public void addPropertyChangedListener(PropertyChangeListener pcl){
		propertyChangeSupport.addPropertyChangeListener(pcl);
	}
	
	/**
	 * Usu listener.
	 * @param pcl PropertyChangeListener
	 */
	public void removePropertyChangedListener(PropertyChangeListener pcl){
		propertyChangeSupport.removePropertyChangeListener(pcl);
	}

}
