package drawingclient;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;

import drawing.NoMoreLeftException;
import drawing.Pen;
import drawing.Queue;

/**
 * 
 * Bean z interfejsem graficznym do obsugi rysowania. Trzy pisaki, dla kadego regulowana
 * grubo linii i kolor, guzik do czyszczenia kartki i wczania/wyczania wygadzania
 * (antialiasing). Generuje zdarzenie PropertyChanged jeli uytkownik zmini ktry z tych
 * parameterw.
 */

public class JPanel_ToolBox extends javax.swing.JPanel{

	/**
	 * Grupa guzikw typu RadioButton do wybierania aktywnego pisaka.
	 */
	private ButtonGroup buttonGroup = new ButtonGroup();

	/**
	 * Radio button do wyboru drugiego pisaka.
	 */
	private JRadioButton pen2RadioButton;

	/**
	 * Radio button do wyboru pierwszego pisaka.
	 */
	private JRadioButton pen1RadioButton;

	/**
	 * CheckBox do ustawiania wygadzania (AA) 
	 */
	private JCheckBox aaCheckBox;

	
	//private boolean reset = false;
	
	/**
	 * Wybrany bean JPanel_PenProporties (s trzy, kady do innego pisaka).
	 */
	private JPanel_PenProporties choosen_PenProporties;

	/**
	 * Konstruktor. Inicjalizacja interfejsu.
	 *
	 */
	public JPanel_ToolBox() {

		try {
			initGUI();

			JPanel_PenProporties1.setColor(Color.BLACK);
			JPanel_PenProporties1.setLineWidth(1);
			JPanel_PenProporties1.addPropertyChangedListener(penChanged);

			JPanel_PenProporties2.setColor(Color.RED);
			JPanel_PenProporties2.setLineWidth(3);
			JPanel_PenProporties2.addPropertyChangedListener(penChanged);

			JPanel_PenProporties3.setColor(Color.WHITE);
			JPanel_PenProporties3.setLineWidth(25);
			JPanel_PenProporties3.addPropertyChangedListener(penChanged);

			setSize(300, 500);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}		

	/**
	 * Inicjalizacja interfejsu.
	 * @throws Exception
	 */
	private void initGUI() throws Exception {
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalStrut(15));

		pen1RadioButton = new JRadioButton();
		pen1RadioButton.setSelected(true);
		choosen_PenProporties = JPanel_PenProporties1;
		buttonGroup.add(pen1RadioButton);
		pen1RadioButton.setText("Pen1");

		pen1RadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				choosen_PenProporties = JPanel_PenProporties1;
				propertyChangeSupport.firePropertyChange("pen",null, choosen_PenProporties.getPen());
			}
		});

		pen2RadioButton = new JRadioButton();
		buttonGroup.add(pen2RadioButton);
		pen2RadioButton.setText("Pen2");

		pen2RadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				choosen_PenProporties = JPanel_PenProporties2;
				propertyChangeSupport.firePropertyChange("pen",null, choosen_PenProporties.getPen());
			}
		});

		add(pen1RadioButton);
		add(JPanel_PenProporties1);
		add(pen2RadioButton);
		add(JPanel_PenProporties2);

		JRadioButton pen3RadioButton = new JRadioButton();
		buttonGroup.add(pen3RadioButton);
		pen3RadioButton.setText("Pen3");
		add(pen3RadioButton);
		
		pen3RadioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {				
					choosen_PenProporties = JPanel_PenProporties3;
					propertyChangeSupport.firePropertyChange("pen",null, choosen_PenProporties.getPen());
			}
		});

		add(JPanel_PenProporties3);
		add(Box.createVerticalGlue());

		JButton clearButton = new JButton();
		clearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				//reset = true;
				propertyChangeSupport.firePropertyChange("clear", false, true);
			}
		});
		clearButton.setText("CLEAR");
		add(clearButton);
		
		aaCheckBox = new JCheckBox();
		aaCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if(aaCheckBox.isSelected())
					propertyChangeSupport.firePropertyChange("aa",false,true);
				else
					propertyChangeSupport.firePropertyChange("aa",true,false);
			}
		});
		aaCheckBox.setSelected(true);
		aaCheckBox.setText("AA");		
		add(aaCheckBox);
		
		add(Box.createVerticalStrut(15));

	}

	/**
	 * Bean do definiowania pierwszego pisaka.
	 */
	private JPanel_PenProporties JPanel_PenProporties1 = new JPanel_PenProporties();

	/**
	 * Bean do definiowania drugiego pisaka.
	 */
	private JPanel_PenProporties JPanel_PenProporties2 = new JPanel_PenProporties();

	/**
	 * Bean do definiowania trzeciego pisaka.
	 */
	private JPanel_PenProporties JPanel_PenProporties3 = new JPanel_PenProporties();

	/**
	 * Daj aktualnie wybrany pisak. 
	 * @return Pisak
	 */
	public Pen getPen() {
		if (pen1RadioButton.isSelected())
			return JPanel_PenProporties1.getPen();
		else if (pen2RadioButton.isSelected())
			return JPanel_PenProporties2.getPen();

		return JPanel_PenProporties3.getPen();
	}

	/**
	 * Czy obrazek ma by wygadzany.
	 * @return Prawda, jeli obrazek ma by wygadzany.
	 */
	public boolean isAntialiased() {
		return aaCheckBox.isSelected();
	}


	
	private class PenChangedListener implements PropertyChangeListener{
		public void propertyChange(PropertyChangeEvent evt) {
			propertyChangeSupport.firePropertyChange("pen", new Pen(), choosen_PenProporties.getPen());						
		}		
	}
	
	/**
	 * Zdarzenie "zmiana pisaka". Podaje pisak wskazany przez uytkownika. 
	 *
	 */
	private PenChangedListener penChanged = new PenChangedListener();
		
	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(
			this);

	/**
	 * Dodaj listener suchajcy zmian pisaka.
	 */
	public void addPropertyChangedListener(PropertyChangeListener pcl) {
		propertyChangeSupport.addPropertyChangeListener(pcl);
	}

	/**
	 * Usu listener.
	 */
	public void removePropertyChangedListener(PropertyChangeListener pcl) {
		propertyChangeSupport.removePropertyChangeListener(pcl);
	}

}
