package drawingserver;

import java.util.HashMap;

import drawing.Curve;
import drawing.NoMoreLeftException;
import drawing.Queue;


/**
 * Reprezentuje klienta (serwer tworzy tyle obiektw klasy Client ilu
 * klientw jest do niego podczonych). Przechowuje informacje o krzywych, jakich klient jeszcze nie posiada a ktre
 * zostan mu wysane przy najbliszej okazji.
 */

public class Client {
	//private HashMap curves = new HashMap();
	/**
	 * Krzywe oczekujce na odebranie przez klienta. S to krzywe ktre narysowa kto
	 * inny a ktrych klient jeszcze nie odebra.
	 */
	private Queue curvesWaiting = new Queue();
	/**
	 * Identyfikator klienta.
	 */
	private String clientID;
	
	/**
	 * Daj unikalny identyfikator klienta (klucz sesji)
	 * @return Unikalny identyfikator klienta (klucz sesji)
	 */
	public String getClientID() {
		return clientID;
	}	

	/**
	 * Stwrz klienta. Klient musi mie unikalny identyfikator.
	 * @param clientID Unikalny identyfikator klienta
	 */
	public Client(String clientID) {		
		this.clientID = clientID;
	}
	
	/**
	 * Dodaj krzyw. Krzywy zostanie dodana do kolejki krzywych oczekujcych na wysanie do klienta.
	 * @param c Krzywa do dodania
	 */
	public void addCurve(Curve c){
		curvesWaiting.add(c);
	}
	
	/**
	 * Usu wszystkie krzywe oczekujce na wysanie. 
	 */
	public void dropAllCurves(){
		curvesWaiting = new Queue();
	}
	
	/**
	 * Daj nastpn krzyw oczekujc na wysanie do klienta.
	 * @return Nastpna krzywa oczekujc na wysanie do klienta.
	 * @throws NoMoreLeftException
	 */
	public Curve pollWaitingCurve() throws NoMoreLeftException{
		try {
			return (Curve)curvesWaiting.poll();
		} catch (ArrayIndexOutOfBoundsException e) {		
			throw new NoMoreLeftException();
		}
	}		

}
