package drawingserver;

import java.util.ArrayList;
import java.util.Hashtable;

import drawing.Curve;
import drawing.NoMoreLeftException;

/**
 * Przechowuje klientw (obiekty klasy Client) i zarzdza nimi. 
 */

public class ClientsBank {
	/**
	 * Struktura hash zawierajca klientw, kluczami s identyfikatory (klucze sesji).
	 */
	private Hashtable clients = new Hashtable();
	
	/**
	 * Lista wszystkich krzywych jakie przeszy przez serwer (jeli kto si doczy to
	 * automatycznie otrzyma wszystko, co do tej pory zostao narysowane.
	 */
	private ArrayList curves = new ArrayList();
	
	/**
	 * Warto do ustalania kolejnych wlonych unikalnych identyfikatorw dla klientw.
	 */
	private int nextFreeClientID = 0; 
	
	
	/**
	 * Konstruktor.
	 *
	 */
	public ClientsBank() {
		super();
		// TODO Auto-generated constructor stub
	}
	
	/**
	 * Usu wszystkie krzywe dla wszystkich klientw (czyszczenie kartki).
	 *
	 */
	public void dropAllCurves(){
		curves = new ArrayList();
		ArrayList cls = new ArrayList(clients.values());
		
			for(int i=0;i<cls.size();i++){
				Client client = (Client)cls.get(i);			
				client.dropAllCurves();
			}
		
	}
	
	/**
	 * Dadaj nowego klienta. Zwraca przyznany uniklany klucz sesji (identyfikator klienta).
	 * @return Unikalny identyfikator klienta
	 */
	public String newClient(){
		String id = "client"+(nextFreeClientID++);
		Client newClient = new Client(id);
		clients.put(id, newClient);
		
		for(int i=0;i<curves.size();i++)
			newClient.addCurve( ((Curve)curves.get(i)) );
		
		System.out.println("New client registered as "+id);
		return id;
	}
	
	/**
	 * Rozelij krzyw do klientw, nie zapisuj na serwerze.
	 * @param c Krzywa
	 */
	public void sendCurveToClients(Curve c){
		ArrayList cls = new ArrayList(clients.values());
		
			for(int i=0;i<cls.size();i++){
				Client client = (Client)cls.get(i);
				if(!client.getClientID().equals(c.getAuthor()))
					client.addCurve(c);
			}
		
	}
	
	/**
	 * Zapisz krzyw i rozelij do klientw.
	 * @param c Krzywa
	 */
	public void saveAndSendCurveToClients(Curve c){
		curves.add(c);
		
		sendCurveToClients(c);
	}
	
	/**
	 * Daj krzyw oczekujc na wysanie do danego klienta.
	 * @param clientID Identyfikator klienta
	 * @return Krzywa
	 * @throws NoMoreLeftException
	 */
	public Curve pollWaitingCurve(String clientID) throws NoMoreLeftException{		
			Client client = (Client)clients.get(clientID);			
			return client.pollWaitingCurve();		
	}
		
}
